@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# Copyright 2005 Messiah College.
+# Copyright 2005-2007 Messiah College.
 # Jason Long <jlong@messiah.edu>
 
 # Copyright (c) 2004 Anthony D. Urso. All rights reserved.
@@ -12,14 +12,18 @@ use warnings;
 
 package Mail::DKIM::Policy;
 
+use Mail::DKIM::DNS;
+
 =head1 NAME
 
-Mail::DKIM::Policy - represents a DKIM sender signing policy
+Mail::DKIM::Policy - represents a DomainKeys Sender Signing Policy record
 
 =head1 DESCRIPTION
 
-A sender signing policy, according to DKIM, is a record published
-in the message author's DNS that describes how they sign messages.
+DomainKeys sender signing policies are described in
+RFC4870(historical). It is a record published in the message
+sender's (i.e. the person who transmitted the message)
+DNS that describes how they sign messages.
 
 =head1 CONSTRUCTORS
 
@@ -27,13 +31,45 @@ in the message author's DNS that describes how they sign messages.
 
   my $policy = Mail::DKIM::Policy->fetch(
                    Protocol => "dns",
-                   Domain => "example.org",
+                   Sender => 'joe@example.org',
                );
 
+The following named arguments are accepted:
+
+=over
+
+=item Protocol
+
+always specify "dns"
+
+=item Author
+
+the "author" of the message for which policy is being checked.
+This is the first email address in the "From" header.
+According to RFC 2822, section 3.6.2, the "From" header lists
+who is responsible for writing the message.
+
+=item Sender
+
+the "sender" of the message for which policy is being checked.
+This is the first email address in the "Sender" header,
+or if there is not a "Sender" header, the "From" header.
+According to RFC 2822, section 3.6.2, the "Sender" header lists
+who is responsible for transmitting the message.
+
+=back
+
+Depending on what type of policy is being checked, both the
+Sender and Author fields may need to be specified.
+
 If a DNS error or timeout occurs, an exception is thrown.
 
-If the policy is not found, or a non-DNS error occurs,
-undef is returned instead of a policy.
+Otherwise, a policy object of some sort will be returned.
+If no policy is actually published,
+then the "default policy" will be returned.
+To check when this happens, use
+
+  my $is_default = $policy->is_implied_default_policy;
 
 =cut
 
@@ -42,51 +78,24 @@ sub fetch
 	my $class = shift;
 	my %prms = @_;
 
-	my $strn;
+	($prms{'Protocol'} eq "dns")
+		or die "invalid protocol '$prms{Protocol}'\n";
 
+	my $host = $class->get_lookup_name(\%prms);
 
-	($prms{'Protocol'} eq "dns") or
-		return;
-
-	my $host = "_policy._domainkey." . $prms{'Domain'};
-
-	use Net::DNS;
-
-	my $rslv = new Net::DNS::Resolver or
-		return;
-	
 	#
 	# perform DNS query for domain policy...
 	#   if the query takes too long, we should catch it and generate
 	#   an error
 	#
-	my $resp;
-	eval
-	{
-		# set a 10 second timeout
-		local $SIG{ALRM} = sub { die "DNS query timeout for $host\n" };
-		alarm 10;
-
-		# the query itself could cause an exception, which would prevent
-		# us from resetting the alarm before leaving the eval {} block
-		# so we wrap the query in a nested eval {} block
-		eval
-		{
-			$resp = $rslv->query($host, "TXT");
-		};
-		my $E = $@;
-		alarm 0;
-		die $E if $E;
-	};
-	my $E = $@;
-	alarm 0; #FIXME- restore previous alarm?
-	die $E if $E;
+	my $resp = Mail::DKIM::DNS::query($host, "TXT");
 	unless ($resp)
 	{
 		# no response => NXDOMAIN, use default policy
-		return $class->new;
+		return $class->default;
 	}
 
+	my $strn;
 	foreach my $ans ($resp->answer) {
 		next unless $ans->type eq "TXT";
 		$strn = join "", $ans->char_str_list;
@@ -95,15 +104,48 @@ sub fetch
 	unless ($strn)
 	{
 		# empty record found in DNS, use default policy
-		return $class->new;
+		return $class->default;
 	}
 
 	return $class->parse(
 			String => $strn,
-			Domain => $prms{Domain}
+			Domain => $prms{Domain},
 			);
 }
 
+# get_lookup_name() - determine name of record to fetch
+#
+sub get_lookup_name
+{
+	my $self = shift;
+	my ($prms) = @_;
+
+	# in DomainKeys, the record to fetch is determined based on the
+	# Sender header, then the From header
+
+	if ($prms->{Author} && !$prms->{Sender})
+	{
+		$prms->{Sender} = $prms->{Author};
+	}
+	if ($prms->{Sender} && !$prms->{Domain})
+	{
+		# pick domain from email address
+		$prms->{Domain} = ($prms->{Sender} =~ /\@([^@]*)$/ and $1);
+	}
+
+	unless ($prms->{Domain})
+	{
+		die "no domain to fetch policy for\n";
+	}
+
+	# IETF seems poised to create policy records this way
+	#my $host = "_policy._domainkey." . $prms{Domain};
+
+	# but Yahoo! policy records are still much more common
+	# see historic RFC4870, section 3.6
+	return "_domainkey." . $prms->{Domain};
+}
+
 =head2 new() - construct a default policy object
 
   my $policy = Mail::DKIM::Policy->new;
@@ -118,7 +160,7 @@ sub new
 
 =head2 parse() - gets a policy object by parsing a string
 
-  my $policy = parse Mail::DKIM::Policy(
+  my $policy = Mail::DKIM::Policy->parse(
                    String => "o=~; t=y"
                );
 
@@ -151,6 +193,15 @@ sub parse
 	return bless \%prms, $class;	
 }
 
+#undocumented private class method
+our $DEFAULT_POLICY;
+sub default
+{
+	my $class = shift;
+	$DEFAULT_POLICY ||= $class->new;
+	return $DEFAULT_POLICY;
+}
+
 =head1 METHODS
 
 =head2 apply() - apply the policy to the results of a DKIM verifier
@@ -186,27 +237,22 @@ sub apply
 	my $self = shift;
 	my ($dkim) = @_;
 
-	my $verify_result = $dkim->result;
 	my $first_party;
-	if ($dkim->message_originator && $dkim->signature)
+	foreach my $signature ($dkim->signatures)
 	{
-		my $oa = $dkim->message_originator->address;
-		my $id = $dkim->signature->identity;
+		next if $signature->result ne "pass";
 
-		if (substr($oa, -length($id)) eq $id)
+		my $oa = $dkim->message_sender->address;
+		if ($signature->identity_matches($oa))
 		{
+			# found a first party signature
 			$first_party = 1;
+			last;
 		}
 	}
 
-	use constant POLICY_NEVER => ".";
-	use constant POLICY_EXCLUSIVE => "!";
-	use constant POLICY_STRONG => "-";
-	return "reject" if ($self->policy eq POLICY_NEVER);
-	return "accept" if ($verify_result eq "pass" && $first_party);
-	return "reject" if ($self->policy eq POLICY_EXCLUSIVE);
-	return "accept" if ($verify_result eq "pass");
-	return "reject" if ($self->policy eq POLICY_STRONG);
+	return "accept" if $first_party;
+	return "reject" if ($self->signall && !$self->testing);
 	return "neutral";
 }
 
@@ -241,6 +287,39 @@ sub flags
 	$self->{tags}->{t};
 }
 
+=head2 is_implied_default_policy() - is this policy implied?
+
+  my $is_implied = $policy->is_implied_default_policy;
+
+If you fetch the policy for a particular domain, but that domain
+does not have a policy published, then the "default policy" is
+in effect. Use this method to detect when that happens.
+
+=cut
+
+sub is_implied_default_policy
+{
+	my $self = shift;
+	my $default_policy = ref($self)->default;
+	return ($self == $default_policy);
+}
+
+=head2 location() - where the policy was fetched from
+
+DomainKeys policies only have per-domain policies, so this will
+be the domain where the policy was published.
+
+If nothing is published for the domain, and the default policy
+was returned instead, the location will be C<undef>.
+
+=cut
+
+sub location
+{
+	my $self = shift;
+	return $self->{Domain};
+}
+
 =head2 note() - get or set the human readable notes (n=) tag
 
 Human readable notes regarding the record. Undef if no notes specified.
@@ -267,18 +346,14 @@ Outbound signing policy for the entity. Possible values are:
 
 =item C<~>
 
-The entity signs some but not all email.
+The default. The domain may sign some (but not all) email.
 
 =item C<->
 
-All mail from the entity is signed; unsigned email MUST NOT be
-accepted, but email signed with a Verifier Acceptable Third
-Party Signature SHOULD be accepted.
+The domain signs all email.
 
 =back
 
-Other values are possible as well, they just haven't been documented yet.
-
 =cut
 
 sub policy
@@ -294,18 +369,18 @@ sub policy
 	}
 	else
 	{
-		return "~";
+		return "~"; # the default
 	}
 }
 
+=head2 signall() - true if policy is "-"
+
+=cut
+
 sub signall
 {
 	my $self = shift;
-
-	$self->policy and $self->policy eq "-" and
-		return 1;
-
-	return;
+	return ($self->policy && $self->policy eq "-");
 }
 
 sub signsome
@@ -341,18 +416,6 @@ sub testing
 
 1;
 
-=head1 BUGS
-
-=over
-
-=item *
-
-If a sender signing policy is not found for a given domain, the
-fetch() method should search the parent domains, according to
-section 4 of the dkim-ssp Internet Draft.
-
-=back
-
 =head1 AUTHOR
 
 Jason Long, E<lt>jlong@messiah.eduE<gt>
