@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 
-# Copyright 2005-2006 Messiah College. All rights reserved.
+# Copyright 2005-2007 Messiah College. All rights reserved.
 # Jason Long <jlong@messiah.edu>
 
 # Copyright (c) 2004 Anthony D. Urso. All rights reserved.
@@ -20,46 +20,53 @@ use Carp;
 
 =head1 NAME
 
-Mail::DKIM::Signature - encapsulates a DKIM signature header
+Mail::DKIM::Signature - represents a DKIM-Signature header
 
 =head1 CONSTRUCTORS
 
 =head2 new() - create a new signature from parameters
 
-  my $signature = new Mail::DKIM::Signature(
+  my $signature = Mail::DKIM::Signature->new(
                       [ Algorithm => "rsa-sha1", ]
                       [ Signature => $base64, ]
                       [ Method => "relaxed", ]
                       [ Domain => "example.org", ]
+                      [ Identity => 'user@example.org', ]
                       [ Headers => "from:subject:date:message-id", ]
                       [ Query => "dns", ]
                       [ Selector => "alpha", ]
+                      [ Timestamp => time(), ]
+                      [ Expiration => time() + 86400, ]
                   );
 
 =cut
 
-sub new {
-	my $type = shift;
+sub new
+{
+	my $class = shift;
 	my %prms = @_;
 	my $self = {};
-	bless $self, $type;
+	bless $self, $class;
 
 	$self->version("1");
 	$self->algorithm($prms{'Algorithm'} || "rsa-sha1");
 	$self->signature($prms{'Signature'});
-	$self->canonicalization($prms{'Method'} || "simple");
+	$self->canonicalization($prms{'Method'}) if exists $prms{'Method'};
 	$self->domain($prms{'Domain'});
 	$self->headerlist($prms{'Headers'});
-	#$self->protocol($prms{'Query'} || "dns");
-	$self->protocol($prms{'Query'} || "dns/txt");
+	$self->protocol($prms{'Query'}) if exists $prms{'Query'};
 	$self->selector($prms{'Selector'});
+	$self->identity($prms{'Identity'}) if exists $prms{'Identity'};
+	$self->timestamp($prms{'Timestamp'}) if defined $prms{'Timestamp'};
+	$self->expiration($prms{'Expiration'}) if defined $prms{'Expiration'};
+	$self->key($prms{'Key'}) if defined $prms{'Key'};
 
 	return $self;
 }
 
 =head2 parse() - create a new signature from a DKIM-Signature header
 
-  my $sig = parse Mail::DKIM::Signature(
+  my $sig = Mail::DKIM::Signature->parse(
                   "DKIM-Signature: a=rsa-sha1; b=yluiJ7+0=; c=relaxed"
             );
 
@@ -95,16 +102,6 @@ sub parse
 	my $self = $class->SUPER::parse($string);
 	$self->{prefix} = $prefix;
 
-	# check version
-	if (my $version = $self->version)
-	{
-		my @ALLOWED_VERSIONS = ("0.5", "1");
-		unless (grep {$_ eq $version} @ALLOWED_VERSIONS)
-		{
-			die "unsupported v=$version tag\n";
-		}
-	}
-
 	return $self;
 }
 
@@ -306,6 +303,102 @@ sub canonicalization
 	}
 }	
 
+use MIME::Base64;
+
+# checks whether this signature specifies a legal canonicalization method
+# returns true if the canonicalization is acceptable, false otherwise
+#
+sub check_canonicalization
+{
+	my $self = shift;
+
+	my ($c1, $c2) = $self->canonicalization;
+
+	my @known = ("nowsp", "simple", "relaxed");
+	return undef unless (grep { $_ eq $c1 } @known);
+	return undef unless (grep { $_ eq $c2 } @known);
+	return 1;
+}
+
+# checks whether the expiration time on this signature is acceptable
+# returns a true value if acceptable, false otherwise
+#
+sub check_expiration
+{
+	my $self = shift;
+	my $x = $self->expiration;
+	return 1 if not defined $x;
+
+	$self->{_verify_time} ||= time();
+	return ($self->{_verify_time} <= $x);
+}
+
+# checks whether the protocol found on this signature is valid for
+# fetching the public key
+# returns a true value if protocol is "dns/txt", false otherwise
+#
+sub check_protocol
+{
+	my $self = shift;
+
+	my ($type, $options) = split(/\//, $self->protocol, 2);
+	return unless ($type eq "dns");
+	return if ($options && $options ne "txt");
+
+	my $v = $self->version;
+	if ($v)
+	{
+		# in v=1 signatures, the /txt option is REQUIRED
+		return unless ($options && $options eq "txt");
+	}
+	return 1;
+}
+
+# checks whether the version tag has an acceptable value
+# returns true if so, otherwise false
+#
+sub check_version
+{
+	my $self = shift;
+
+	# check version
+	if (my $version = $self->version)
+	{
+		my @ALLOWED_VERSIONS = ("0.5", "1");
+		return (grep {$_ eq $version} @ALLOWED_VERSIONS);
+	}
+
+	# we still consider a missing v= tag acceptable,
+	# for backwards-compatibility
+	return 1;
+}
+
+=head2 data() - get or set the signature data (b=) field
+
+  my $base64 = $signature->data;
+  $signature->data($base64);
+
+The signature data. Whitespace is automatically stripped from the
+returned value. The data is Base64-encoded.
+
+=cut
+
+sub data
+{
+	my $self = shift;
+
+	if (@_)
+	{
+		$self->set_tag("b", shift);
+	}
+
+	my $b = $self->get_tag("b");
+	$b =~ tr/\015\012 \t//d  if defined $b;
+	return $b;
+}	
+
+*signature = \*data;
+
 =head2 domain() - get or set the domain (d=) field
 
   my $d = $signature->domain;          # gets the domain value
@@ -314,6 +407,10 @@ sub canonicalization
 The domain of the signing entity, as specified in the signature.
 This is the domain that will be queried for the public key.
 
+If using an "internationalized domain name", the domain name must be
+converted to ASCII (following section 4.1 of RFC 3490) before passing
+it to this method.
+
 =cut
 
 sub domain
@@ -347,41 +444,6 @@ sub expiration
 	return $self->get_tag("x");
 }
 
-use MIME::Base64;
-
-sub check_canonicalization
-{
-	my $self = shift;
-
-	my ($c1, $c2) = $self->canonicalization;
-
-	my @known = ("nowsp", "simple", "relaxed");
-	return undef unless (grep { $_ eq $c1 } @known);
-	return undef unless (grep { $_ eq $c2 } @known);
-	return 1;
-}
-
-# checks whether the protocol found on this subject is valid for
-# fetching the public key
-# returns a true value if protocol is "dns/txt", false otherwise
-#
-sub check_protocol
-{
-	my $self = shift;
-
-	my ($type, $options) = split(/\//, $self->protocol, 2);
-	return unless ($type eq "dns");
-	return if ($options && $options ne "txt");
-
-	my $v = $self->version;
-	if ($v)
-	{
-		# in v=1 signatures, the /txt option is REQUIRED
-		return unless ($options && $options eq "txt");
-	}
-	return 1;
-}
-
 # allows the type of signature to determine what "algorithm" gets used
 sub get_algorithm_class
 {
@@ -403,28 +465,36 @@ sub get_algorithm_class
 Public key to fetch is determined by the protocol, selector, and domain
 fields.
 
+This method caches the result of the fetch, so subsequent calls will not
+require additional DNS queries.
+
+This method will C<die> if an error occurs.
+
 =cut
 
 sub get_public_key
 {
 	my $self = shift;
 
-	unless ($self->{public})
+	unless (exists $self->{public})
 	{
+		# this ensures we only try fetching once, even if an error occurs
+		$self->{public} = undef;
+
 		my $pubk = Mail::DKIM::PublicKey->fetch(
 			Protocol => $self->protocol,
 			Selector => $self->selector,
 			Domain => $self->domain);
 		unless ($pubk)
 		{
-			die "no public key available\n";
+			die "not available\n";
 		}
 
 		if ($pubk->revoked)
 		{
 			# FIXME- the key was checked in fetch(), so if the
 			# key was really revoked, we shouldn't have gotten here
-			die "public key: revoked\n";
+			die "revoked\n";
 		}
 
 		$self->{public} = $pubk;
@@ -513,6 +583,10 @@ header, or the Sender: header, but this is not required to have a
 valid signature. Whether the identity used is "authorized" to sign
 for the given message is not determined here.
 
+If using an "internationalized domain name", the domain name must be
+converted to ASCII (following section 4.1 of RFC 3490) before passing
+it to this method.
+
 =cut
 
 sub identity
@@ -551,6 +625,25 @@ sub identity_matches
 	return lc($addr) eq lc($id);
 }
 
+=head2 key() - get or set the key object
+
+  my $key = $signature->key;
+
+  $signature->key(Mail::DKIM::PrivateKey->load(File => "private.key"));
+
+=cut
+
+sub key
+{
+	my $self = shift;
+	if (@_)
+	{
+		$self->{Key} = shift;
+		$self->{KeyFile} = undef;
+	}
+	return $self->{Key};
+}
+
 =head2 method() - get or set the canonicalization (c=) field
 
 Message canonicalization (default is "simple"). This informs the verifier
@@ -567,7 +660,7 @@ sub method
 		$self->set_tag("c", shift);
 	}
 
-	return lc $self->get_tag("c");
+	return (lc $self->get_tag("c")) || "simple";
 }	
 
 =head2 protocol() - get or set the query methods (q=) field
@@ -651,32 +744,6 @@ sub selector {
 	return $self->get_tag("s");
 }	
 
-=head2 data() - get or set the signature data (b=) field
-
-  my $base64 = $signature->data;
-  $signature->data($base64);
-
-The signature data. Whitespace is automatically stripped from the
-returned value. The data is Base64-encoded.
-
-=cut
-
-sub data
-{
-	my $self = shift;
-
-	if (@_)
-	{
-		$self->set_tag("b", shift);
-	}
-
-	my $b = $self->get_tag("b");
-	$b =~ tr/\015\012 \t//d  if defined $b;
-	return $b;
-}	
-
-*signature = \*data;
-
 =head2 prettify() - alters the signature to look "nicer" as an email header
 
   $signature->prettify;
@@ -685,6 +752,8 @@ This method may alter the signature in a way that breaks signatures, so
 it should be done ONLY when the signature is being generated, BEFORE being
 fed to the canonicalization algorithm.
 
+See also prettify_safe(), which will not break signatures.
+
 =cut
 
 sub prettify
@@ -757,6 +826,10 @@ sub version
 	return $self->get_tag("v");
 }
 
+=head1 SEE ALSO
+
+L<Mail::DKIM::DkSignature> for DomainKey-Signature headers
+
 =head1 AUTHOR
 
 Jason Long, E<lt>jlong@messiah.eduE<gt>
