@@ -28,7 +28,7 @@ Mail::DKIM::Signer - generates a DKIM signature for a message
                   Method => "relaxed",
                   Domain => "example.org",
                   Selector => "selector1",
-                  KeyFile => "private.key");
+                  KeyFile => "private.key",
              );
 
   # read an email from a file handle
@@ -65,23 +65,39 @@ Mail::DKIM::Signer - generates a DKIM signature for a message
   # create a signer using a custom policy
   my $dkim = Mail::DKIM::Signer->new(
                   Policy => $policyfn,
-                  KeyFile => "private.key",
              );
 
-You must always specify the name of a private key file. In addition,
-you must specify a policy object, or specify the algorithm, method,
-domain, and selector to use. Use of the policy object lets you defer
-the determination of algorithm, method, domain and selector until
-the message being signed has been partially read.
+The "default policy" is to create a DKIM signature using the specified
+parameters, but only if the message's sender matches the domain.
+The following parameters can be passed to this new() method to
+influence the resulting signature:
+Algorithm, Method, Domain, Selector, KeyFile, Identity, Timestamp.
 
-See Mail::DKIM::SignerPolicy for more information about policy objects.
+If you want different behavior, you can provide a "signer policy"
+instead. A signer policy is a subroutine or class that determines
+signature parameters after the message's headers have been parsed.
+See the section L</"SIGNER POLICIES"> below for more information.
+
+See L<Mail::DKIM::SignerPolicy> for more information about policy objects.
+
+In addition to the parameters demonstrated above, the following
+are recognized:
+
+=over
+
+=item Key
+
+rather than using C<KeyFile>, use C<Key> to use an already-loaded
+L<Mail::DKIM::PrivateKey> object.
+
+=back
 
 =cut
 
 package Mail::DKIM::Signer;
 use base "Mail::DKIM::Common";
 use Carp;
-our $VERSION = '0.26';
+our $VERSION = 0.31;
 
 # PROPERTIES
 #
@@ -107,15 +123,15 @@ our $VERSION = '0.26';
 # $dkim->{Selector}
 #   identifies name of the selector identifying the key
 #
+# $dkim->{Key}
+#   the loaded private key
+#
 # private:
 #
 # $dkim->{algorithms} = []
 #   an array of algorithm objects... an algorithm object is created for
 #   each signature being added to the message
 #
-# $dkim->{private}
-#   the loaded private key
-#
 # $dkim->{result}
 #   result of the signing policy: "signed" or "skipped"
 #
@@ -130,14 +146,9 @@ sub init
 
 	if (defined $self->{KeyFile})
 	{
-		croak "not a file: " . $self->{KeyFile}
-			unless (-f $self->{KeyFile});
-
-		$self->{private} = Mail::DKIM::PrivateKey->load(
+		$self->{Key} ||= Mail::DKIM::PrivateKey->load(
 				File => $self->{KeyFile});
 	}
-	croak "No private key specified"
-		unless ($self->{private});
 	
 	unless ($self->{"Algorithm"})
 	{
@@ -213,12 +224,18 @@ sub finish_header
 		}
 
 		$self->add_signature(
-			new Mail::DKIM::Signature(
+			Mail::DKIM::Signature->new(
 				Algorithm => $self->{"Algorithm"},
 				Method => $self->{"Method"},
 				Headers => $self->headers,
 				Domain => $self->{"Domain"},
 				Selector => $self->{"Selector"},
+				Key => $self->{"Key"},
+				KeyFile => $self->{"KeyFile"},
+				($self->{"Identity"} ?
+					(Identity => $self->{"Identity"}) : ()),
+				($self->{"Timestamp"} ?
+					(Timestamp => $self->{"Timestamp"}) : ()),
 			));
 	}
 
@@ -242,14 +259,28 @@ sub finish_body
 		# finished canonicalizing
 		$algorithm->finish_body;
 
+		# load the private key file if necessary
+		my $signature = $algorithm->signature;
+		my $key = $signature->{Key}
+			|| $signature->{KeyFile}
+			|| $self->{Key}
+			|| $self->{KeyFile};
+		if (not ref $key)
+		{
+			$key = Mail::DKIM::PrivateKey->load(
+					File => $key);
+		}
+		$key
+			or die "no key available to sign with\n";
+
 		# compute signature value
-		my $signb64 = $algorithm->sign($self->{private});
-		$algorithm->signature->data($signb64);
+		my $signb64 = $algorithm->sign($key);
+		$signature->data($signb64);
 
 		# insert linebreaks in signature data, if desired
-		$algorithm->signature->prettify_safe();
+		$signature->prettify_safe();
 
-		$self->{signature} = $algorithm->signature;
+		$self->{signature} = $signature;
 		$self->{result} = "signed";
 	}
 }
@@ -277,7 +308,7 @@ and generates a signature.
 
 Signer policies can use this method to specify complete parameters for
 the signature to add, including what type of signature. For more information,
-see Mail::DKIM::SignerPolicy.
+see L<Mail::DKIM::SignerPolicy>.
 
 =cut
 
@@ -288,7 +319,7 @@ sub add_signature
 
 	# create a canonicalization filter and algorithm
 	my $algorithm_class = $signature->get_algorithm_class(
-			$self->{"Algorithm"});
+			$signature->algorithm);
 	my $algorithm = $algorithm_class->new(
 			Signature => $signature,
 			Debug_Canonicalization => $self->{Debug_Canonicalization},
@@ -395,6 +426,50 @@ sub want_header
 	return scalar grep { lc($_) eq lc($header_name) } @DEFAULT_HEADERS;
 }
 
+=head2 key() - get or set the private key object
+
+  my $key = $dkim->key;
+
+  $dkim->key(Mail::DKIM::PrivateKey->load(File => "private.key"));
+
+If you use this method to specify a private key,
+do not use L</"key_file()">.
+
+=cut
+
+sub key
+{
+	my $self = shift;
+	if (@_)
+	{
+		$self->{Key} = shift;
+		$self->{KeyFile} = undef;
+	}
+	return $self->{Key};
+}
+
+=head2 key_file() - get or set the filename containing the private key
+
+  my $filename = $dkim->key_file;
+
+  $dkim->key_file("private.key");
+
+If you use this method to specify a private key file,
+do not use L</"key()">.
+
+=cut
+
+sub key_file
+{
+	my $self = shift;
+	if (@_)
+	{
+		$self->{Key} = undef;
+		$self->{KeyFile} = shift;
+	}
+	return $self->{KeyFile};
+}
+
 =head2 method() - get or set the selected canonicalization method
 
   $alg = $dkim->method;
@@ -417,22 +492,29 @@ sub method
 
   my $address = $dkim->message_originator;
 
-Returns the "originator address" found in the message. This is typically
-the (first) name and email address found in the From: header. The returned
-object is of type Mail::Address. To get just the email address part, do:
+Returns the "originator address" found in the message, as a
+L<Mail::Address> object.
+This is typically the (first) name and email address found in the
+From: header. If there is no From: header,
+then an empty L<Mail::Address> object is returned.
+
+To get just the email address part, do:
 
   my $email = $dkim->message_originator->address;
 
+See also L</"message_sender()">.
 
 =head2 message_sender() - access the "From" or "Sender" header
 
   my $address = $dkim->message_sender;
 
-Returns the "sender" found in the message. This is typically the (first)
-name and email address found in the Sender: header. If there is no Sender:
-header, it is the first name and email address in the From: header.
-The returned object is of type Mail::Address, so to get just the email
-address part, do:
+Returns the "sender" found in the message, as a L<Mail::Address> object.
+This is typically the (first) name and email address found in the
+Sender: header. If there is no Sender: header, it is the first name and
+email address in the From: header. If neither header is present,
+then an empty L<Mail::Address> object is returned.
+
+To get just the email address part, do:
 
   my $email = $dkim->message_sender->address;
 
@@ -470,6 +552,10 @@ Returns the generated signature. The signature is an object of type
 Mail::DKIM::Signature. If multiple signatures were generated, this method
 returns the last one.
 
+The signature should be B<prepended> to the message to make the
+resulting message. At the very least, it should precede any headers
+that were signed.
+
 =head2 signatures() - access list of generated signature objects
 
   my @signatures = $dkim->signatures;
@@ -511,7 +597,8 @@ created. E.g.,
 Or the policy object can actually create the signature, using the
 add_signature method within the policy object.
 If you add a signature, you do not need to return a nonzero value.
-This mechanism can be utilized to create multiple signatures.
+This mechanism can be utilized to create multiple signatures,
+or to create the older DomainKey-style signatures.
 
   my $policyfn = sub {
       my $dkim = shift;
@@ -523,6 +610,14 @@ This mechanism can be utilized to create multiple signatures.
                       Domain => "example.org",
                       Selector => "mx1",
               ));
+      $dkim->add_signature(
+              new Mail::DKIM::DkSignature(
+                      Algorithm => "rsa-sha1",
+                      Method => "nofws",
+                      Headers => $dkim->headers,
+                      Domain => "example.org",
+                      Selector => "mx1",
+              ));
       return;
   };
 
@@ -532,7 +627,7 @@ specified in the new() constructor.
 
 =head1 SEE ALSO
 
-Mail::DKIM::SignerPolicy
+L<Mail::DKIM::SignerPolicy>
 
 =head1 AUTHOR
 
